# Generated by Django 2.2.2 on 2019-07-01 17:44
from zipfile import ZipFile

from django.db import migrations

def set_document_revision_version(apps, schema_editor):
    # Set the version number found in the zip file.
    DocumentRevision = apps.get_model('document', 'DocumentRevision')
    revisions = DocumentRevision.objects.all()
    for revision in revisions:
        if not revision.file_object:
            revision.delete()
            continue
        zip = ZipFile(revision.file_object.path, 'r')
        version = zip.read('filetype-version')
        revision.doc_version = float(version)
        revision.save()

class Migration(migrations.Migration):

    dependencies = [
        ('document', '0010_merge_20190701_1737'),
    ]

    operations = [
        migrations.RunPython(set_document_revision_version),
    ]



# Generated by Django 2.1.7 on 2019-03-16 02:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('style', '0001_squashed_0002_auto_20151226_1110'),
    ]

    operations = [
        migrations.AlterField(
            model_name='citationstyle',
            name='contents',
            field=models.TextField(default='', help_text='The XML style definiton.'),
        ),
        migrations.AlterField(
            model_name='citationstyle',
            name='short_title',
            field=models.SlugField(default='default', help_text='A title used for constant names.', max_length=40),
        ),
        migrations.AlterField(
            model_name='citationstyle',
            name='title',
            field=models.CharField(default='Default', help_text='The human readable title.', max_length=128),
        ),
        migrations.AlterField(
            model_name='documentstyle',
            name='contents',
            field=models.TextField(default='', help_text='The CSS style definiton.'),
        ),
        migrations.AlterField(
            model_name='documentstyle',
            name='filename',
            field=models.SlugField(default='default', help_text='The base of the filenames the style occupies.', max_length=20),
        ),
        migrations.AlterField(
            model_name='documentstyle',
            name='title',
            field=models.CharField(default='Default', help_text='The human readable title.', max_length=128),
        ),
    ]



from builtins import str
from builtins import object

import os
import uuid
from PIL import Image as PilImage
from io import BytesIO

from django.db import models
from django.contrib.auth.models import User
from django.db import IntegrityError
from django.core.files.uploadedfile import SimpleUploadedFile
from document.models import Document

ALLOWED_FILETYPES = ['image/jpeg', 'image/png', 'image/svg+xml']
ALLOWED_EXTENSIONS = ['jpeg', 'jpg', 'png', 'svg']


def get_file_path(instance, filename):
    ext = filename.split('.')[-1].lower()
    if ext not in ALLOWED_EXTENSIONS:
        raise IntegrityError
    filename = "%s.%s" % (uuid.uuid4(), ext)
    return os.path.join('images', filename)


class Image(models.Model):
    uploader = models.ForeignKey(
        User,
        related_name='image_uploader',
        on_delete=models.deletion.CASCADE
    )
    added = models.DateTimeField(auto_now_add=True)
    image = models.FileField(upload_to=get_file_path)
    thumbnail = models.ImageField(
        upload_to='image_thumbnails',
        max_length=500,
        blank=True,
        null=True)
    file_type = models.CharField(max_length=20, blank=True, null=True)
    height = models.IntegerField(blank=True, null=True)
    width = models.IntegerField(blank=True, null=True)
    checksum = models.BigIntegerField(default=0)

    def __str__(self):
        return str(self.pk)

    def is_deletable(self):
        reverse_relations = [
            f for f in self._meta.model._meta.get_fields()
            if (f.one_to_many or f.one_to_one) and
            f.auto_created and not f.concrete
        ]

        for r in reverse_relations:
            if r.remote_field.model.objects.filter(
                **{r.field.name: self}
            ).exists():
                return False
        return True

    def create_checksum(self):
        if not self.image:
            return
        if self.checksum == 0:
            from time import time
            if hasattr(self.image.file, 'size'):
                self.checksum = int(
                    str(self.image.file.size) + str(time()).split('.')[0])
            else:
                self.checksum = time()

    def check_filetype(self):
        if not self.image:
            return
        if not hasattr(self.image.file, 'content_type'):
            return

        if self.image.file.content_type not in ALLOWED_FILETYPES:
            raise IntegrityError

    def create_thumbnail(self):
        # original code for this method came from
        # http://snipt.net/danfreak/generate-thumbnails-in-django-with-pil/

        # If there is no image associated with this.
        # do not create thumbnail
        if not self.image:
            return
        if not hasattr(self.image.file, 'content_type'):
            return

        # Set our max thumbnail size in a tuple (max width, max height)
        # THUMBNAIL_SIZE = (170,200)

        DJANGO_TYPE = self.image.file.content_type

        if DJANGO_TYPE == 'image/jpeg':
            PIL_TYPE = 'jpeg'
            FILE_EXTENSION = 'jpg'
            self.file_type = DJANGO_TYPE
        elif DJANGO_TYPE == 'image/png':
            PIL_TYPE = 'png'
            FILE_EXTENSION = 'png'
            self.file_type = DJANGO_TYPE
        else:
            self.file_type = DJANGO_TYPE
            return

        # Open original photo which we want to thumbnail using PIL's Image
        image = PilImage.open(BytesIO(self.image.read()))

        self.width, self.height = image.size

        # cropping the thumbnail to exactly 60 x 60 px
        src_width, src_height = image.size
        dst_width = dst_height = 60

        if src_width < src_height:
            crop_width = crop_height = src_width
            x_offset = 0
            y_offset = int((src_height - crop_height)/2)
        else:
            crop_width = crop_height = src_height
            x_offset = int((src_width - crop_width)/2)
            y_offset = 0

        image = image.crop(
            (x_offset,
             y_offset,
             x_offset +
             int(crop_width),
                y_offset +
                int(crop_height)))

        # Convert to RGB if necessary
        # Thanks to Limodou on DjangoSnippets.org
        # http://www.djangosnippets.org/snippets/20/
        #
        # I commented this part since it messes up my png files
        #
        # if image.mode not in ('L', 'RGB'):
        #    image = image.convert('RGB')

        # We use our PIL Image object to create the thumbnail, which already
        # has a thumbnail() convenience method that contrains proportions.
        # Additionally, we use Image.ANTIALIAS to make the image look better.
        # Without antialiasing the image pattern artifacts may result.
        image.thumbnail((dst_width, dst_height), PilImage.ANTIALIAS)

        # Save the thumbnail
        temp_handle = BytesIO()
        image.save(temp_handle, PIL_TYPE)
        temp_handle.seek(0)

        # Save image to a SimpleUploadedFile which can be saved into
        # ImageField
        suf = SimpleUploadedFile(os.path.split(
            self.image.name)[-1], temp_handle.read(), content_type=DJANGO_TYPE)
        # Save SimpleUploadedFile into image field
        self.thumbnail.save(
            '%s_thumbnail.%s' %
            (os.path.splitext(
                suf.name)[0],
                FILE_EXTENSION),
            suf,
            save=False)

    def save(self, *args, **kwargs):
        # create a thumbnail
        self.create_checksum()
        self.check_filetype()
        self.create_thumbnail()

        super(Image, self).save(*args, **kwargs)


# Image linked to a particular User.
class UserImage(models.Model):
    title = models.CharField(max_length=128)
    owner = models.ForeignKey(
        User,
        related_name='image_owner',
        blank=True,
        null=True,
        on_delete=models.deletion.CASCADE
    )
    image_cat = models.CharField(max_length=255, default='')
    image = models.ForeignKey(
        Image,
        on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        if len(self.title) > 0:
            return self.title
        else:
            return str(self.pk)


# Image linked to a document
class DocumentImage(models.Model):
    title = models.CharField(max_length=128, default='')
    document = models.ForeignKey(
        Document,
        on_delete=models.deletion.CASCADE
    )
    image = models.ForeignKey(
        Image,
        on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        if len(self.title) > 0:
            return self.title
        else:
            return str(self.pk)


# category
class ImageCategory(models.Model):
    category_title = models.CharField(max_length=100)
    category_owner = models.ForeignKey(
        User,
        on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        return self.category_title

    class Meta(object):
        verbose_name_plural = 'Image categories'



# Generated by Django 2.2.3 on 2019-08-03 05:49
from django.db import migrations, models


class Migration(migrations.Migration):
    
    dependencies = [
        ('style', '0002_auto_20190316_0241'),
    ]

    operations = [
        migrations.AlterField(
            model_name='citationstyle',
            name='contents',
            field=models.TextField(help_text='The style definiton (JSON, accepts also XML).'),
        ),
        migrations.AlterField(
            model_name='citationlocale',
            name='contents',
            field=models.TextField(help_text='The locale definiton (JSON, accepts also XML).'),
        )
    ]



# Generated by Django 2.1.7 on 2019-02-27 21:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0006_auto_20190118_1342'),
    ]

    operations = [
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('write', 'Writer'), ('write-tracked', 'Write with tracked changes'), ('comment', 'Commentator'), ('review', 'Reviewer'), ('read', 'Reader'), ('read-without-comments', 'Reader without comment access')], max_length=21),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-30 04:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0016_auto_20190829_2147'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documenttemplate',
            name='citation_styles',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-29 19:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0015_auto_20190828_1108'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.1, max_digits=3),
        ),
        migrations.AlterField(
            model_name='documentrevision',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.1, max_digits=3),
        ),
        migrations.AlterField(
            model_name='documenttemplate',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.1, max_digits=3),
        ),
    ]



from django.http import JsonResponse
from django.core.exceptions import ValidationError
from django.contrib.auth.decorators import login_required
from document.helpers.serializers import PythonWithURLSerializer
from django.utils.translation import ugettext as _

from .models import DocumentStyle, DocumentStyleFile, ExportTemplate
from document.models import DocumentTemplate


@login_required
def delete_document_style(request):
    response = {}
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse(
            response,
            status=405
        )
    id = int(request.POST['id'])
    if request.user.is_staff:
        document_style = DocumentStyle.objects.filter(id=id).first()
    else:
        document_style = DocumentStyle.objects.filter(
            id=id,
            document_template__user=request.user
        ).first()
    if not document_style:
        return JsonResponse(
            response,
            status=405
        )
    if document_style.document_template.documentstyle_set.all().count() < 2:
        # We do not the style if there is only one left for this template
        response['errors'] = {
            'template': _('The template needs at least 1 document style')
        }
        return JsonResponse(
            response,
            status=400
        )
    document_style.delete()
    status = 200
    return JsonResponse(
        response,
        status=status
    )


@login_required
def save_document_style(request):
    response = {}
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse(
            response,
            status=405
        )
    template_id = int(request.POST['template_id'])
    if request.user.is_staff:
        template = DocumentTemplate.objects.filter(id=template_id).first()
    else:
        template = DocumentTemplate.objects.filter(
            id=template_id,
            user=request.user
        ).first()
    if not template:
        return JsonResponse(
            response,
            status=405
        )
    id = int(request.POST['id'])
    if id > 0:
        document_style = DocumentStyle.objects.filter(
            id=id,
            document_template=template
        ).first()
        status = 200
    else:
        document_style = DocumentStyle()
        document_style.document_template = template
        status = 201
    if not document_style:
        return JsonResponse(
            response,
            status=405
        )
    document_style.title = request.POST['title']
    document_style.slug = request.POST['slug']
    document_style.contents = request.POST['contents']
    try:
        document_style.full_clean()
        document_style.save()
    except ValidationError as e:
        response['errors'] = e.message_dict
        return JsonResponse(
            response,
            status=400
        )
    deleted_files = request.POST.getlist('deleted_files[]')
    added_files = request.FILES.getlist('added_files[]')
    for file in added_files:
        dsf = DocumentStyleFile()
        dsf.file = file
        dsf.style = document_style
        dsf.save()
    for file in deleted_files:
        dsf = DocumentStyleFile.objects.filter(
            style=document_style,
            filename=file
        ).first()
        if dsf:
            dsf.delete()
    serializer = PythonWithURLSerializer()
    response['doc_style'] = serializer.serialize(
        [document_style],
        use_natural_foreign_keys=True,
        fields=['title', 'slug', 'contents', 'documentstylefile_set']
    )
    return JsonResponse(
        response,
        status=status
    )


@login_required
def delete_export_template(request):
    response = {}
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse(
            response,
            status=405
        )
    id = int(request.POST['id'])
    if request.user.is_staff:
        export_template = ExportTemplate.objects.filter(id=id).first()
    else:
        export_template = ExportTemplate.objects.filter(
            id=id,
            document_template__user=request.user
        ).first()
    if not export_template:
        return JsonResponse(
            response,
            status=405
        )
    export_template.delete()
    status = 200
    return JsonResponse(
        response,
        status=status
    )


@login_required
def save_export_template(request):
    response = {}
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse(
            response,
            status=405
        )
    template_id = int(request.POST['template_id'])
    if request.user.is_staff:
        template = DocumentTemplate.objects.filter(id=template_id).first()
    else:
        template = DocumentTemplate.objects.filter(
            id=template_id,
            user=request.user
        ).first()
    if not template:
        return JsonResponse(
            response,
            status=405
        )
    id = int(request.POST['id'])
    if id > 0:
        export_template = ExportTemplate.objects.filter(
            id=id,
            document_template=template
        ).first()
        status = 200
    else:
        export_template = ExportTemplate()
        export_template.document_template = template
        status = 201
    if not export_template:
        return JsonResponse(
            response,
            status=405
        )
    export_template.template_file = request.FILES['added_file']
    export_template.file_type = request.POST['added_file_type']
    try:
        export_template.full_clean()
        export_template.save()
    except ValidationError as e:
        response['errors'] = e.message_dict
        return JsonResponse(
            response,
            status=400
        )
    serializer = PythonWithURLSerializer()
    response['export_template'] = serializer.serialize(
        [export_template],
        fields=['file_type', 'template_file', 'title']
    )
    return JsonResponse(
        response,
        status=status
    )



# Generated by Django 2.2.3 on 2019-08-08 09:26

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0012_auto_20190704_1219'),
    ]

    operations = [
        migrations.AddField(
            model_name='documenttemplate',
            name='added',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='documenttemplate',
            name='updated',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-04 12:19

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0011_auto_20190701_1744'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document.DocumentTemplate'),
        ),
        migrations.CreateModel(
            name='AccessRightInvite',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=254)),
                ('rights', models.CharField(choices=[('write', 'Writer'), ('write-tracked', 'Write with tracked changes'), ('comment', 'Commentator'), ('review', 'Reviewer'), ('read', 'Reader'), ('read-without-comments', 'Reader without comment access')], max_length=21)),
                ('document', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document.Document')),
            ],
        ),
    ]



# Generated by Django 2.2.2 on 2019-07-01 17:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0009_documentrevision_doc_version'),
        ('document', '0008_auto_20190701_0943'),
    ]

    operations = [
    ]



import time
import os
import bleach
from tornado.escape import json_decode, json_encode
from django.core import serializers
from django.http import HttpResponse, JsonResponse, HttpRequest
from django.contrib.auth.decorators import login_required
from django.db import transaction
from django.core.files import File
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _
from django.conf import settings
from django.core.mail import send_mail
from django.db.models import Q
from django.contrib.admin.views.decorators import staff_member_required

from user.util import get_user_avatar_url
from document.models import Document, AccessRight, DocumentRevision, \
    DocumentTemplate, AccessRightInvite, CAN_UPDATE_DOCUMENT, \
    FW_DOCUMENT_VERSION
from usermedia.models import DocumentImage, Image
from bibliography.models import Entry
from document.helpers.serializers import PythonWithURLSerializer
from bibliography.views import serializer
from style.models import DocumentStyle, ExportTemplate
from base.html_email import html_email
from user.models import TeamMember


@login_required
def get_documentlist_extra(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        ids = request.POST['ids'].split(',')
        docs = Document.objects.filter(Q(owner=request.user) | Q(
            accessright__user=request.user)).filter(id__in=ids)
        response['documents'] = []
        for doc in docs:
            images = {}
            for image in doc.documentimage_set.all():
                images[image.image.id] = {
                    'added': image.image.added,
                    'checksum': image.image.checksum,
                    'file_type': image.image.file_type,
                    'height': image.image.height,
                    'id': image.image.id,
                    'image': image.image.image.url,
                    'thumbnail': image.image.thumbnail.url,
                    'title': image.title,
                    'width': image.image.width
                }
            response['documents'].append({
                'images': images,
                'contents': doc.contents,
                'comments': doc.comments,
                'bibliography': doc.bibliography,
                'id': doc.id
            })
    return JsonResponse(
        response,
        status=status
    )


@login_required
def documents_list(request):
    documents = Document.objects.filter(
        Q(owner=request.user) | Q(accessright__user=request.user),
        listed=True
    ).order_by('-updated')
    output_list = []
    for document in documents:
        if document.owner == request.user:
            access_right = 'write'
        else:
            access_right = AccessRight.objects.get(
                user=request.user,
                document=document
            ).rights
        revisions = DocumentRevision.objects.filter(document=document)
        revision_list = []
        for revision in revisions:
            revision_list.append({
                'date': time.mktime(revision.date.utctimetuple()),
                'note': revision.note,
                'file_name': revision.file_name,
                'pk': revision.pk
            })

        added = time.mktime(document.added.utctimetuple())
        updated = time.mktime(document.updated.utctimetuple())
        is_owner = False
        if document.owner == request.user:
            is_owner = True
        output_list.append({
            'id': document.id,
            'title': document.title,
            'is_owner': is_owner,
            'owner': {
                'id': document.owner.id,
                'name': document.owner.readable_name,
                'avatar': get_user_avatar_url(document.owner)
            },
            'added': added,
            'updated': updated,
            'rights': access_right,
            'revisions': revision_list
        })
    return output_list


@login_required
def get_access_rights(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        ar_qs = AccessRight.objects.filter(document__owner=request.user)
        in_qs = AccessRightInvite.objects.filter(document__owner=request.user)
        doc_ids = request.POST.getlist('document_ids[]')
        if len(doc_ids) > 0:
            ar_qs = ar_qs.filter(document_id__in=doc_ids)
            in_qs = in_qs.filter(document_id__in=doc_ids)
        access_rights = []
        for ar in ar_qs:
            access_rights.append({
                'document_id': ar.document.id,
                'user_id': ar.user.id,
                'user_name': ar.user.readable_name,
                'rights': ar.rights,
                'avatar': get_user_avatar_url(ar.user)
            })
        response['access_rights'] = access_rights
        invites = []
        for inv in in_qs:
            invites.append({
                'document_id': inv.document.id,
                'email': inv.email,
                'rights': inv.rights
            })
        response['invites'] = invites
    return JsonResponse(
        response,
        status=status
    )


@login_required
@transaction.atomic
def save_access_rights(request):
    status = 405
    response = {}
    if request.is_ajax() and request.method == 'POST':
        doc_ids = json_decode(request.POST['document_ids'])
        rights = json_decode(request.POST['access_rights'])
        invites = json_decode(request.POST['invites'])
        for doc_id in doc_ids:
            doc = Document.objects.filter(
                pk=doc_id,
                owner=request.user
            ).first()
            if not doc:
                continue
            for right in rights:
                if right['rights'] == 'delete':
                    # Status 'delete' means the access right is marked for
                    # deletion.
                    AccessRight.objects.filter(
                        document_id=doc_id,
                        user_id=right['user_id']
                    ).delete()
                else:
                    access_right = AccessRight.objects.filter(
                        document_id=doc_id,
                        user_id=right['user_id']
                    ).first()
                    if access_right:
                        if access_right.rights != right['rights']:
                            access_right.rights = right['rights']
                            send_share_notification(
                                request,
                                doc_id,
                                right['user_id'],
                                right['rights'],
                                True
                            )
                    else:
                        access_right = AccessRight.objects.create(
                            document_id=doc_id,
                            user_id=right['user_id'],
                            rights=right['rights']
                        )
                        send_share_notification(
                            request,
                            doc_id,
                            right['user_id'],
                            right['rights'],
                            False
                        )
                    access_right.save()
            for invite in invites:
                if invite['rights'] == 'delete':
                    # Status 'delete' means the invite is marked for
                    # deletion.
                    AccessRightInvite.objects.filter(
                        document_id=doc_id,
                        email=invite['email']
                    ).delete()
                else:
                    old_invite = AccessRightInvite.objects.filter(
                        document_id=doc_id,
                        email=invite['email']
                    ).first()
                    if old_invite:
                        if old_invite.rights != invite['rights']:
                            old_invite.rights = invite['rights']
                            old_invite.save()
                            send_invite_notification(
                                request,
                                doc_id,
                                invite['email'],
                                invite['rights'],
                                old_invite,
                                True
                            )
                    else:
                        new_invite = AccessRightInvite.objects.create(
                            document_id=doc_id,
                            email=invite['email'],
                            rights=invite['rights']
                        )
                        new_invite.save()
                        send_invite_notification(
                            request,
                            doc_id,
                            invite['email'],
                            invite['rights'],
                            new_invite,
                            False
                        )
        status = 201
    return JsonResponse(
        response,
        status=status
    )


def apply_invite(inv, user):
    old_ar = AccessRight.objects.filter(
        user=user,
        document=inv.document
    ).first()
    if old_ar:
        # If the user already has rights, we should only be upgrading
        # them, not downgrade. Unfortuantely it is not easy to
        # say how each right compares. So unless the invite gives read
        # access, or the user already has write access, we change to
        # the access right of the invite.
        if inv.rights == 'read':
            pass
        elif old_ar.rights == 'write':
            pass
        else:
            old_ar.rights = inv.rights
            old_ar.save()
    elif inv.document.owner == user:
        pass
    else:
        ar = AccessRight.objects.create(
            document=inv.document,
            user=user,
            rights=inv.rights
        )
        ar.save()
        if not TeamMember.objects.filter(
            leader=inv.document.owner,
            member=user
        ).first():
            tm1 = TeamMember.objects.create(
                leader=inv.document.owner,
                member=user
            )
            tm1.save()
        if not TeamMember.objects.filter(
            leader=user,
            member=inv.document.owner,
        ).first():
            tm2 = TeamMember.objects.create(
                leader=user,
                member=inv.document.owner
            )
            tm2.save()
    inv.delete()


@login_required
def invite(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        id = int(request.POST['id'])
        inv = AccessRightInvite.objects.filter(id=id).first()
        if inv:
            response['redirect'] = inv.document.get_absolute_url()
            apply_invite(inv, request.user)
        else:
            response['redirect'] = ''
    return JsonResponse(
        response,
        status=status
    )


@login_required
def get_documentlist(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        response['documents'] = documents_list(request)
        response['team_members'] = []
        for team_member in request.user.leader.all():
            tm_object = {}
            tm_object['id'] = team_member.member.id
            tm_object['name'] = team_member.member.readable_name
            tm_object['username'] = team_member.member.get_username()
            '''
            tm_object['avatar'] = get_user_avatar_url(
                team_member.member
            )['url']
            '''
            tm_object['avatar'] = get_user_avatar_url(team_member.member)
            response['team_members'].append(tm_object)
        serializer = PythonWithURLSerializer()
        export_temps = serializer.serialize(
            ExportTemplate.objects.filter(
                Q(document_template__user=None) |
                Q(document_template__user=request.user)
            ),
            fields=['file_type', 'template_file', 'title']
        )
        response['export_templates'] = [obj['fields'] for obj in export_temps]
        doc_styles = serializer.serialize(
            DocumentStyle.objects.filter(
                Q(document_template__user=None) |
                Q(document_template__user=request.user)
            ),
            use_natural_foreign_keys=True,
            fields=['title', 'slug', 'contents', 'documentstylefile_set']
        )
        response['document_styles'] = [obj['fields'] for obj in doc_styles]
        doc_templates = DocumentTemplate.objects.filter(
            Q(user=request.user) | Q(user=None)
        )
        response['document_templates'] = [
            {'id': obj.id, 'title': obj.title} for obj in doc_templates
        ]

    return JsonResponse(
        response,
        status=status
    )


@login_required
def delete(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        doc_id = int(request.POST['id'])
        document = Document.objects.get(pk=doc_id, owner=request.user)
        if document.is_deletable():
            image_ids = list(
                DocumentImage.objects.filter(document_id=doc_id)
                .values_list('image_id', flat=True)
            )
            document.delete()
            for image in Image.objects.filter(id__in=image_ids):
                if image.is_deletable():
                    image.delete()
            response['done'] = True
        else:
            response['done'] = False
    return JsonResponse(
        response,
        status=status
    )


def send_share_notification(request, doc_id, collaborator_id, rights, change):
    owner = request.user.readable_name
    document = Document.objects.get(id=doc_id)
    collaborator = User.objects.get(id=collaborator_id)
    collaborator_name = collaborator.readable_name
    collaborator_email = collaborator.email
    document_title = document.title
    if len(document_title) == 0:
        document_title = _('Untitled')
    link = HttpRequest.build_absolute_uri(request, document.get_absolute_url())
    if change:
        message_text = _(
            ('Hey %(collaborator_name)s,\n%(owner)s has changed your access '
             'rights to %(rights)s on the document \'%(document_title)s\'. '
             '\nAccess the document through this link: %(link)s')
        ) % {
            'owner': owner,
            'rights': rights,
            'collaborator_name': collaborator_name,
            'link': link,
            'document_title': document_title
        }
        body_html_intro = _(
            ('<p>Hey %(collaborator_name)s,<br>%(owner)s has changed your '
             'access rights to %(rights)s on the document '
             '\'%(document_title)s\'.</p>')
        ) % {
            'owner': owner,
            'rights': rights,
            'collaborator_name': collaborator_name,
            'document_title': document_title
        }
    else:
        message_text = _(
            ('Hey %(collaborator_name)s,\n%(owner)s has shared the document '
             '\'%(document_title)s\' with you and given you %(rights)s access '
             'rights. '
             '\nAccess the document through this link: %(link)s')
        ) % {
            'owner': owner,
            'rights': rights,
            'collaborator_name': collaborator_name,
            'link': link,
            'document_title': document_title
        }
        body_html_intro = _(
            ('<p>Hey %(collaborator_name)s,<br>%(owner)s has shared the '
             'document \'%(document_title)s\' with you and given you '
             '%(rights)s access rights.</p>')
        ) % {
            'owner': owner,
            'rights': rights,
            'collaborator_name': collaborator_name,
            'document_title': document_title
        }

    body_html = (
        '<h1>%(document_title)s %(shared)s</h1>'
        '%(body_html_intro)s'
        '<table>'
        '<tr><td>'
        '%(Document)s'
        '</td><td>'
        '<b>%(document_title)s</b>'
        '</td></tr>'
        '<tr><td>'
        '%(Author)s'
        '</td><td>'
        '%(owner)s'
        '</td></tr>'
        '<tr><td>'
        '%(AccessRights)s'
        '</td><td>'
        '%(rights)s'
        '</td></tr>'
        '</table>'
        '<div class="actions"><a class="button" href="%(link)s">'
        '%(AccessTheDocument)s'
        '</a></div>'
    ) % {
        'shared': _('shared'),
        'body_html_intro': body_html_intro,
        'Document': _('Document'),
        'document_title': document_title,
        'Author': _('Author'),
        'owner': owner,
        'AccessRights': _('Access Rights'),
        'rights': rights,
        'link': link,
        'AccessTheDocument': _('Access the document')
    }
    send_mail(
        _('Document shared:') +
        ' ' +
        document_title,
        message_text,
        settings.DEFAULT_FROM_EMAIL,
        [collaborator_email],
        fail_silently=True,
        html_message=html_email(body_html)
    )


def send_invite_notification(request, doc_id, email, rights, invite, change):
    owner = request.user.readable_name
    document = Document.objects.get(id=doc_id)
    document_title = document.title
    if len(document_title) == 0:
        document_title = _('Untitled')
    link = HttpRequest.build_absolute_uri(request, invite.get_absolute_url())
    if change:
        message_text = _(
            ('Hey %(email)s,\nas we told you previously, %(owner)s has '
             'invited you to join Fidus Writer and shared the document '
             '\'%(document_title)s\' with you. '
             '\n%(owner)s has now changed your access rights to %(rights)s.'
             '\nAccess the document through this link: %(link)s')
        ) % {
            'owner': owner,
            'rights': rights,
            'email': email,
            'link': link,
            'document_title': document_title
        }
        body_html_intro = _(
            ('<p>Hey %(email)s,<br>as we told you previously, '
             '%(owner)s has invited you to join Fidus Writer and shared the '
             '\'%(document_title)s\' with you.</p>'
             '<p>%(owner)s has now changed your access rights to %(rights)s. '
             '</p>')
        ) % {
            'owner': owner,
            'rights': rights,
            'email': email,
            'document_title': document_title
        }
    else:
        message_text = _(
            ('Hey %(email)s,\n%(owner)s has invited you to Fidus '
             ' Writer, shared the document \'%(document_title)s\' with you, '
             'and given you %(rights)s access rights. '
             '\nAccess the document through this link: %(link)s')
        ) % {
            'owner': owner,
            'rights': rights,
            'email': email,
            'link': link,
            'document_title': document_title
        }
        body_html_intro = _(
            ('<p>Hey %(email)s,<br>%(owner)s has invited you to '
             'Fidus Writer, shared the document \'%(document_title)s\' with '
             'you, and given you %(rights)s access rights.</p>')
        ) % {
            'owner': owner,
            'rights': rights,
            'email': email,
            'document_title': document_title
        }

    body_html = (
        '<h1>%(document_title)s %(shared)s</h1>'
        '%(body_html_intro)s'
        '<table>'
        '<tr><td>'
        '%(Document)s'
        '</td><td>'
        '<b>%(document_title)s</b>'
        '</td></tr>'
        '<tr><td>'
        '%(Author)s'
        '</td><td>'
        '%(owner)s'
        '</td></tr>'
        '<tr><td>'
        '%(AccessRights)s'
        '</td><td>'
        '%(rights)s'
        '</td></tr>'
        '</table>'
        '<div class="actions"><a class="button" href="%(link)s">'
        '%(AccessTheDocument)s'
        '</a></div>'
    ) % {
        'shared': _('shared'),
        'body_html_intro': body_html_intro,
        'Document': _('Document'),
        'document_title': document_title,
        'Author': _('Author'),
        'owner': owner,
        'AccessRights': _('Access Rights'),
        'rights': rights,
        'link': link,
        'AccessTheDocument': _('Sign up or log in and access the document')
    }
    send_mail(
        _('Document shared:') +
        ' ' +
        document_title,
        message_text,
        settings.DEFAULT_FROM_EMAIL,
        [email],
        fail_silently=True,
        html_message=html_email(body_html)
    )


@login_required
def create_doc(request, template_id):
    response = {}
    status = 405
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse(
            response,
            status=status
        )
    document_template = DocumentTemplate.objects.filter(
        Q(user=request.user) | Q(user=None),
        id=template_id
    ).first()
    if not document_template:
        return JsonResponse(
            response,
            status=status
        )
    document = Document.objects.create(
        owner_id=request.user.pk,
        template_id=template_id
    )
    status = 201
    response['id'] = document.id
    return JsonResponse(
        response,
        status=status
    )


@login_required
def import_create(request):
    # First step of import: Create a document and return the id of it
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 201
        import_id = request.POST['import_id']
        document_template = DocumentTemplate.objects.filter(
            Q(user=request.user) | Q(user=None),
            import_id=import_id
        ).first()
        if not document_template:
            title = request.POST['template_title']
            definition = json_encode(json_decode(request.POST['template']))
            document_template = DocumentTemplate()
            document_template.title = title
            document_template.import_id = import_id
            document_template.user = request.user
            document_template.definition = definition
            document_template.save()
        document = Document.objects.create(
            owner_id=request.user.pk,
            template=document_template
        )
        response['id'] = document.id
    return JsonResponse(
        response,
        status=status
    )


@login_required
def import_image(request):
    # create an image for a document
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        document = Document.objects.filter(
            owner_id=request.user.pk,
            id=int(request.POST['doc_id'])
        ).first()
        if document:
            status = 201
        else:
            status = 401
            return JsonResponse(
                response,
                status=status
            )
        checksum = request.POST['checksum']
        image = Image.objects.filter(checksum=checksum).first()
        if image is None:
            image = Image.objects.create(
                uploader=request.user,
                image=request.FILES['image'],
                checksum=checksum
            )
        doc_image = DocumentImage.objects.create(
            image=image,
            title=request.POST['title'],
            document=document
        )
        response['id'] = doc_image.image.id
    return JsonResponse(
        response,
        status=status
    )


@login_required
def import_doc(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        doc_id = request.POST['id']
        # There is a doc_id, so we overwrite an existing doc rather than
        # creating a new one.
        document = Document.objects.get(id=int(doc_id))
        if (
            document.owner != request.user and
            len(AccessRight.objects.filter(
                document_id=doc_id,
                user_id=request.user.id,
                rights__in=CAN_UPDATE_DOCUMENT
            )) == 0
        ):
            response['error'] = 'No access to file'
            status = 403
            return JsonResponse(
                response,
                status=status
            )
        document.title = request.POST['title']
        # We need to decode/encode the following so that it has the same
        # character encoding as used the the save_document method in ws_views.
        document.contents = json_encode(json_decode(request.POST['contents']))
        document.comments = json_encode(json_decode(request.POST['comments']))
        document.bibliography = \
            json_encode(json_decode(request.POST['bibliography']))
        # document.doc_version should always be the current version, so don't
        # bother about it.
        document.save()
        response['document_id'] = document.id
        response['added'] = time.mktime(document.added.utctimetuple())
        response['updated'] = time.mktime(document.updated.utctimetuple())
        status = 200

    return JsonResponse(
        response,
        status=status
    )


@login_required
def upload_revision(request):
    response = {}
    can_save = False
    status = 405
    if request.is_ajax() and request.method == 'POST':
        document_id = request.POST['document_id']
        document = Document.objects.filter(id=int(document_id)).first()
        if document:
            if document.owner == request.user:
                can_save = True
            else:
                access_rights = AccessRight.objects.filter(
                    document=document, user=request.user)
                if len(access_rights) > 0 and access_rights[
                    0
                ].rights == 'write':
                    can_save = True
        if can_save:
            status = 201
            revision = DocumentRevision()
            revision.file_object = request.FILES['file']
            revision.file_name = request.FILES['file'].name
            revision.note = request.POST['note']
            revision.document_id = document_id
            revision.save()
    return JsonResponse(
        response,
        status=status
    )


# Download a revision that was previously uploaded
@login_required
def get_revision(request, revision_id):
    if request.user.is_staff:
        revision = DocumentRevision.objects.get(pk=int(revision_id))
    else:
        revision = DocumentRevision.objects.get(
            pk=int(revision_id),
            document__owner=request.user
        )
    http_response = HttpResponse(
        revision.file_object.file,
        content_type='application/zip; charset=x-user-defined',
        status=200
    )
    http_response[
        'Content-Disposition'] = 'attachment; filename=some_name.zip'
    return http_response


@login_required
def delete_revision(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        revision_id = request.POST['id']
        revision = DocumentRevision.objects.filter(pk=int(revision_id)).first()
        if revision:
            document = revision.document
            if document.owner == request.user:
                status = 200
                revision.delete()
    return JsonResponse(
        response,
        status=status
    )


# Check doc access rights.
def has_doc_access(doc, user):
    if doc.owner == user:
        return True
    access_rights = AccessRight.objects.filter(
        document=doc,
        user=user
    ).first()
    if access_rights:
        return True
    else:
        return False


@login_required
def comment_notify(request):
    response = {}
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse(
            response,
            status=405
        )
    doc_id = request.POST['doc_id']
    collaborator_id = request.POST['collaborator_id']
    comment_text = request.POST['comment_text']
    comment_html = bleach.clean(
        request.POST['comment_html'],
        strip=True
    )
    notification_type = request.POST['type']
    collaborator = User.objects.filter(pk=collaborator_id).first()
    document = Document.objects.filter(pk=doc_id).first()
    if (
        not document or
        not collaborator or
        not comment_text or
        not comment_html or
        not has_doc_access(document, request.user) or
        not notification_type
    ):
        return JsonResponse(
            response,
            status=403
        )
    if (
        not has_doc_access(document, collaborator)
    ):
        # Tagged user has no access to document and will therefore not be
        # notified
        return JsonResponse(
            response,
            status=200
        )
    commentator = request.user.readable_name
    collaborator_name = collaborator.readable_name
    collaborator_email = collaborator.email
    document_title = document.title
    if len(document_title) == 0:
        document_title = _('Untitled')
    link = HttpRequest.build_absolute_uri(request, document.get_absolute_url())

    if notification_type == 'mention':

        message_text = _(
            ('Hey %(collaborator_name)s,\n%(commentator)s has mentioned you '
             'in a comment in the document \'%(document)s\':'
             '\n\n%(comment_text)s'
             '\n\nGo to the document here: %(link)s')
        ) % {
               'commentator': commentator,
               'collaborator_name': collaborator_name,
               'link': link,
               'document': document_title,
               'comment_text': comment_text
        }

        body_html_title = _(
            ('Hey %(collaborator_name)s,<br>%(commentator)s has mentioned '
             'you in a comment in the document \'%(document_title)s\'.')
        ) % {
            'commentator': commentator,
            'collaborator_name': collaborator_name,
            'document_title': document_title
        }
        message_title = _('Comment on :') + ' ' + document_title
    else:
        message_text = _(
            ('Hey %(collaborator_name)s,\n%(commentator)s has assigned you to '
             'a comment in the document \'%(document)s\':\n\n%(comment_text)s'
             '\n\nGo to the document here: %(link)s')
        ) % {
               'commentator': commentator,
               'collaborator_name': collaborator_name,
               'link': link,
               'document': document_title,
               'comment_text': comment_text
        }
        body_html_title = _(
            ('Hey %(collaborator_name)s,<br>%(commentator)s has assigned you '
             'to a comment in the document \'%(document_title)s\'.')
        ) % {
            'commentator': commentator,
            'collaborator_name': collaborator_name,
            'document_title': document_title
        }
        message_title = _('Comment assignment on :') + ' ' + document_title

    body_html = _(
        ('<p>Hey %(collaborator_name)s,<br>%(commentator)s has assigned '
         'you to a comment in the document \'%(document)s\':</p>'
         '%(comment_html)s'
         '<p>Go to the document <a href="%(link)s">here</a>.</p>')
    ) % {
        'commentator': commentator,
        'collaborator_name': collaborator_name,
        'link': link,
        'document': document_title,
        'comment_html': comment_html
    }

    body_html = (
        '<h1>%(body_html_title)s</h1>'
        '<table>'
        '<tr><td>'
        '%(Document)s'
        '</td><td>'
        '<b>%(document_title)s</b>'
        '</td></tr>'
        '<tr><td>'
        '%(Author)s'
        '</td><td>'
        '%(commentator)s'
        '</td></tr>'
        '<tr><td>'
        '%(Comment)s'
        '</td><td>'
        '%(comment_html)s'
        '</td></tr>'
        '</table>'
        '<div class="actions"><a class="button" href="%(link)s">'
        '%(AccessTheDocument)s'
        '</a></div>'
    ) % {
        'body_html_title': body_html_title,
        'Document': _('Document'),
        'document_title': document_title,
        'Author': _('Author'),
        'commentator': commentator,
        'Comment': _('Comment'),
        'comment_html': comment_html,
        'link': link,
        'AccessTheDocument': _('Access the document')
    }

    send_mail(
        message_title,
        message_text,
        settings.DEFAULT_FROM_EMAIL,
        [collaborator_email],
        fail_silently=True,
        html_message=html_email(body_html)
    )
    return JsonResponse(
        response,
        status=200
    )


# maintenance views
@staff_member_required
def get_all_old_docs(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        doc_list = Document.objects.filter(
            doc_version__lt=str(FW_DOCUMENT_VERSION)
        )[:10]
        response['docs'] = serializers.serialize(
            'json',
            doc_list
        )
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def save_doc(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        doc_id = request.POST['id']
        doc = Document.objects.get(pk=int(doc_id))
        # Only looking at fields that may have changed.
        contents = request.POST.get('contents', False)
        bibliography = request.POST.get('bibliography', False)
        comments = request.POST.get('comments', False)
        last_diffs = request.POST.get('last_diffs', False)
        version = request.POST.get('version', False)
        if contents:
            doc.contents = contents
        if bibliography:
            doc.bibliography = bibliography
        if comments:
            doc.comments = comments
        if version:
            doc.version = version
        if last_diffs:
            doc.last_diffs = last_diffs
        doc.doc_version = FW_DOCUMENT_VERSION
        doc.save()
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def get_user_biblist(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        user_id = request.POST['user_id']
        response['bibList'] = serializer.serialize(
            Entry.objects.filter(
                entry_owner_id=user_id
            ), fields=(
                    'entry_key',
                    'entry_owner',
                    'bib_type',
                    'fields'
            )
        )
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def get_all_template_ids(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        templates = DocumentTemplate.objects.filter(
            doc_version__lt=str(FW_DOCUMENT_VERSION)
        ).only('id')
        response["template_ids"] = []
        for template in templates:
            response["template_ids"].append(template.id)
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def get_template(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        template_id = request.POST['id']
        template = DocumentTemplate.objects.filter(pk=int(template_id)).first()
        if template:
            status = 200
            response['definition'] = template.definition
            response['title'] = template.title
            response['doc_version'] = template.doc_version
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def get_template_extras(request):
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse({}, status=405)
    id = request.POST['id']
    doc_template = DocumentTemplate.objects.filter(
        id=id
    ).first()
    status = 200
    if doc_template is None:
        return JsonResponse({}, status=405)
    serializer = PythonWithURLSerializer()
    export_templates = serializer.serialize(
        doc_template.exporttemplate_set.all()
    )
    document_styles = serializer.serialize(
        doc_template.documentstyle_set.all(),
        use_natural_foreign_keys=True,
        fields=['title', 'slug', 'contents', 'documentstylefile_set']
    )
    response = {
        'export_templates': export_templates,
        'document_styles': document_styles,
    }
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def save_template(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        template_id = request.POST['id']
        template = DocumentTemplate.objects.filter(pk=int(template_id)).first()
        if template:
            status = 200
            # Only looking at fields that may have changed.
            definition = request.POST.get('definition', False)
            if definition:
                template.definition = definition
            template.doc_version = FW_DOCUMENT_VERSION
            template.save()
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def get_all_revision_ids(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        revisions = DocumentRevision.objects.filter(
            doc_version__lt=str(FW_DOCUMENT_VERSION)
        ).only('id')
        response["revision_ids"] = []
        for revision in revisions:
            response["revision_ids"].append(revision.id)
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def update_revision(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        revision_id = request.POST['id']
        revision = DocumentRevision.objects.filter(pk=int(revision_id)).first()
        if revision:
            status = 200
            # keep the filename
            file_name = revision.file_object.name.split('/')[-1]
            # Delete the FieldFile as otherwise the file remains.
            revision.file_object.delete()
            revision.file_object = request.FILES['file']
            revision.file_object.name = file_name
            revision.doc_version = FW_DOCUMENT_VERSION
            revision.save()
    return JsonResponse(
        response,
        status=status
    )


@staff_member_required
def add_images_to_doc(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 201
        doc_id = request.POST['doc_id']
        doc = Document.objects.get(id=doc_id)
        # Delete all existing image links
        DocumentImage.objects.filter(
            document_id=doc_id
        ).delete()
        ids = request.POST.getlist('ids[]')
        for id in ids:
            title = 'Deleted'
            image = Image.objects.filter(id=id).first()
            if image:
                user_image = image.userimage_set.all().first()
                if user_image:
                    title = user_image.title
            else:
                image = Image()
                image.pk = id
                image.uploader = doc.owner
                f = open(os.path.join(
                    settings.PROJECT_PATH, "base/static/img/error.png"
                ))
                image.image.save('error.png', File(f))
                image.save()
            DocumentImage.objects.create(
                document=doc,
                image=image,
                title=title
            )

    return JsonResponse(
        response,
        status=status
    )



import os

PORT = 8000

# If you want to show debug messages, set DEBUG to True.
DEBUG = True

# This determines whether the server is used for testing and will let the
# users know upon signup know that their documents may disappear.
TEST_SERVER = True
# This is the contact email that will be shown in various places all over
# the site.
CONTACT_EMAIL = 'mail@email.com'
# If websockets is running on a non-standard port, add it here:
WS_PORT = False

ADMINS = (
    ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

# The top path of the project. Depending on whether ./manage.py is executed or
# the fiduswriter pip package, it is either the dir that contains manage.py or
# the cwd.
PROJECT_PATH = os.environ.get('PROJECT_PATH')
# SRC_PATH is the root path of the FW sources.
SRC_PATH = os.environ.get('SRC_PATH')

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_PATH, 'fiduswriter.sql'),
        'CONN_MAX_AGE': 15
    }
}

# Will let any user not delete more than 5000 bibliography entries
# simultaneously
DATA_UPLOAD_MAX_NUMBER_FIELDS = 5000

# Whether anyone surfing to the site can open an account.
REGISTRATION_OPEN = True

# This determines whether there is a star labeled "Free" on the login page
IS_FREE = True

# Send emails to console.
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

# Or send emails using an SMTP server
# EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
# EMAIL_HOST = 'localhost'
# EMAIL_HOST_USER = ''
# EMAIL_HOST_PASSWORD = ''
# EMAIL_PORT = 25
# EMAIL_SUBJECT_PREFIX = '[Fidus Writer]'


# Local time zone for this installation. Keep UTC here, the frontend will
# translate this to the correct local time.
TIME_ZONE = 'UTC'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, the server will make some optimizations so as not
# to load the internationalization machinery. Fidus Writer makes use of
# internationalization, so you should probably keep this on.

USE_I18N = True

LOCALE_PATHS = (
    os.path.join(SRC_PATH, 'locale'),
)

# A list of allowed hostnames of this Fidus Writer installation
ALLOWED_HOSTS = [
    'localhost',
]

# If you set this to False, the server will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, the server will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# The default is the media folder in the directory above this file.
MEDIA_ROOT = os.path.join(PROJECT_PATH, 'media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = '/tmp/django-static/'
# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    os.path.join(PROJECT_PATH, 'static-transpile'),
    os.path.join(PROJECT_PATH, 'static-libs'),
)

LOGIN_URL = '/'

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    # 'django.contrib.staticfiles.finders.DefaultStorageFinder',
    'compressor.finders.CompressorFinder',
)

COMPRESS_OUTPUT_DIR = '.'

# Make this unique, and don't share it with anybody. Change the default string.
SECRET_KEY = '2ouq2zgw5y-@w+t6!#zf#-z1inigg7$lg3p%8e3kkob1bf$#p4'


# These middleware classes is what Fidus Writer needs in its standard setup.
# You only need to change this in very advanced setups.
BASE_MIDDLEWARE = [
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.locale.LocaleMiddleware',
]

MIDDLEWARE = []


# The location of the top urls.py file inside the core folder.
# You only need to change this in very advanced setups.
ROOT_URLCONF = 'core.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(PROJECT_PATH, 'templates'),
            # Put strings here, like "/home/html/django_templates".
            # You only need to change this in very advanced setups.
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.request',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


# The following are the apps needed by Fidus Writer.

BASE_INSTALLED_APPS = [
    'base',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.flatpages',
    'django_js_error_hook',
    'npm_mjs',
    'fixturemedia',
    'browser_check',
    'menu',
    'document',
    'bibliography',
    'usermedia',
    'user',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'avatar',
    'compressor',
    'feedback',
    'style'
]
# These are additional apps to be overriden by configuration.py
INSTALLED_APPS = []

# These are apps that are removed from base apps overriden by configuration.py
REMOVED_APPS = []


AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",

    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend",
)

TEST_RUNNER = 'django.test.runner.DiscoverRunner'


# Define available languages
# You only need to change this in very advanced setups.
def gettext(s): return s


LANGUAGES = (
    ('en', gettext('English')),
    ('bg', gettext('Bulgarian')),
    ('de', gettext('German')),
    ('fr', gettext('French')),
    ('it', gettext('Italian')),
    ('es', gettext('Spanish')),
    ('pt-br', gettext('Portuguese (Brazil)')),
)


LOGIN_REDIRECT_URL = '/'

ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_EMAIL_VERIFICATION = "mandatory"

# allow login either with email or username
ACCOUNT_AUTHENTICATION_METHOD = "username_email"
ACCOUNT_ADAPTER = "user.adapter.AccountAdapter"

AUTH_PROFILE_MODULE = "account.UserProfile"

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'formatters': {
        'verbose': {
            'format': (
                '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d '
                '%(message)s'
            )
        },
        'simple': {
            'format': '\033[22;32m%(levelname)s\033[0;0m %(message)s'
        },
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        'null': {
            'class': 'logging.NullHandler',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'django.security.DisallowedHost': {
            'handlers': ['null'],
            'propagate': False,
        },
        'javascript_error': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'tornado.access': {
            'handlers': ['null'],
            'propagate': False,
        }
    }
}

# Global variables for avatar
# You only need to change this in very advanced setups.
AVATAR_GRAVATAR_BACKUP = False
AVATAR_DEFAULT_URL = 'img/default_avatar.png'
AVATAR_MAX_AVATARS_PER_USER = 1

WEBSOCKET_PING_INTERVAL = 55

ADMIN_SITE_TITLE = gettext('Fidus Writer Admin')
ADMIN_SITE_HEADER = gettext('Fidus Writer Administration Site')
ADMIN_INDEX_TITLE = gettext('Welcome to the Fidus Writer Administration Site')

# The below allow login via JavaScript
SESSION_COOKIE_HTTPONLY = False
CSRF_COOKIE_HTTPONLY = False

# To make npm-mjs enable the webpack offline plugin
WEBPACK_CONFIG_TEMPLATE = os.path.join(
    os.path.dirname(
        os.path.realpath(__file__)
    ),
    'webpack.config.template.js'
)



from builtins import map
from builtins import range
import errno
import os
import socket
import sys
import threading
from unittest import skipIf         # NOQA: Imported here for backward compatibility

from django.core.exceptions import ImproperlyConfigured
from django.db import connections
from django.utils import six
from django.test.testcases import TransactionTestCase

from tornado.ioloop import IOLoop

from base.servers.tornado_django_hybrid import make_tornado_server

try:
    from asyncio import set_event_loop_policy
    from tornado.platform.asyncio import AnyThreadEventLoopPolicy
    set_event_loop_policy(AnyThreadEventLoopPolicy())
except ImportError:
    pass


class LiveTornadoThread(threading.Thread):
    """
    Thread for running a live http server while the tests are running.
    """

    def __init__(self,
                 host,
                 possible_ports,
                 static_handler,
                 connections_override=None):
        self.host = host
        self.port = None
        self.possible_ports = possible_ports
        self.is_ready = threading.Event()
        self.error = None
        self.static_handler = static_handler
        self.connections_override = connections_override
        super(LiveTornadoThread, self).__init__()

    def run(self):
        """
        Sets up the live server and databases, and then loops over handling
        http requests.
        """
        if self.connections_override:
            # Override this thread's database connections with the ones
            # provided by the main thread.
            for alias, conn in list(self.connections_override.items()):
                connections[alias] = conn
        try:
            self.httpd = make_tornado_server()

            # Go through the list of possible ports, hoping that we can find
            # one that is free to use for the WSGI server.
            for index, port in enumerate(self.possible_ports):
                try:
                    self.httpd.listen(int(port))
                except socket.error as e:
                    if (index + 1 < len(self.possible_ports) and
                            e.errno == errno.EADDRINUSE):
                        # This port is already in use, so we go on and try with
                        # the next one in the list.
                        continue
                    else:
                        # Either none of the given ports are free or the error
                        # is something else than "Address already in use". So
                        # we let that error bubble up to the main thread.
                        raise
                else:
                    # A free port was found.
                    self.port = port
                    break

            self.is_ready.set()
            self.ioloop = IOLoop.current()
            self.ioloop.start()
        except Exception as e:
            self.error = e
            self.is_ready.set()
            raise

    def terminate(self):
        if hasattr(self, 'httpd'):
            self.httpd.stop()

        if hasattr(self, 'ioloop'):
            self.ioloop.stop()


class LiveTornadoTestCase(TransactionTestCase):
    """
    Does basically the same as TransactionTestCase but also launches a live
    http server in a separate thread so that the tests may use another testing
    framework, such as Selenium for example, instead of the built-in dummy
    client.
    Note that it inherits from TransactionTestCase instead of TestCase because
    the threads do not share the same transactions (unless if using in-memory
    sqlite) and each thread needs to commit all their transactions so that the
    other thread can see the changes.
    """

    static_handler = None
    live_server_url = None

    @classmethod
    def setUpClass(cls):
        connections_override = {}
        for conn in connections.all():
            # If using in-memory sqlite databases, pass the connections to
            # the server thread.
            if (conn.vendor == 'sqlite' and
                    conn.settings_dict['NAME'] == ':memory:'):
                # Explicitly enable thread-shareability for this connection
                conn.allow_thread_sharing = True
                connections_override[conn.alias] = conn

        # Launch the live server's thread
        specified_address = os.environ.get(
            'DJANGO_LIVE_TEST_SERVER_ADDRESS', 'localhost:8081-8140')
        # The specified ports may be of the form '8000-8010,8080,9200-9300'
        # i.e. a comma-separated list of ports or ranges of ports, so we break
        # it down into a detailed list of all possible ports.
        possible_ports = []
        try:
            host, port_ranges = specified_address.split(':')
            for port_range in port_ranges.split(','):
                # A port range can be of either form: '8000' or '8000-8010'.
                extremes = list(map(int, port_range.split('-')))
                assert len(extremes) in [1, 2]
                if len(extremes) == 1:
                    # Port range of the form '8000'
                    possible_ports.append(extremes[0])
                else:
                    # Port range of the form '8000-8010'
                    for port in range(extremes[0], extremes[1] + 1):
                        possible_ports.append(port)
        except Exception:
            msg = 'Invalid address ("%s") for live server.' % specified_address
            six.reraise(
                ImproperlyConfigured,
                ImproperlyConfigured(msg),
                sys.exc_info()[2]
            )
        cls.server_thread = LiveTornadoThread(
            host,
            possible_ports,
            cls.static_handler,
            connections_override=connections_override
        )
        cls.server_thread.daemon = True
        cls.server_thread.start()

        # Wait for the live server to be ready
        cls.server_thread.is_ready.wait()
        if cls.server_thread.error:
            # Clean up behind ourselves, since tearDownClass won't get called
            # in case of errors.
            cls._tearDownClassInternal()
            raise cls.server_thread.error

        cls.live_server_url = 'http://%s:%s' % (
            cls.server_thread.host, cls.server_thread.port)

        super(LiveTornadoTestCase, cls).setUpClass()

    @classmethod
    def _tearDownClassInternal(cls):
        # There may not be a 'server_thread' attribute if setUpClass() for some
        # reasons has raised an exception.
        if hasattr(cls, 'server_thread'):
            # Terminate the live server's thread
            cls.server_thread.terminate()
        # Restore sqlite connections' non-shareability
        for conn in connections.all():
            if (conn.vendor == 'sqlite' and
                    conn.settings_dict['NAME'] == ':memory:'):
                conn.allow_thread_sharing = False

    @classmethod
    def tearDownClass(cls):
        cls._tearDownClassInternal()
        super(LiveTornadoTestCase, cls).tearDownClass()



# Generated by Django 2.2.3 on 2019-08-11 10:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0013_auto_20190808_1126'),
        ('style', '0006_auto_20190809_1757'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documenttemplate',
            name='document_styles',
        ),
        migrations.RemoveField(
            model_name='documenttemplate',
            name='export_templates',
        ),
        migrations.DeleteModel(
            name='ExportTemplate',
        ),
    ]



import os
import tempfile
import shutil
import urllib

from django.core.management.commands import makemessages
from django.core.management import call_command
from django.core.management.utils import popen_wrapper

# This makes makemessages create both translations for Python and JavaScript
# code in one go.
#
# Additionally, it works around xgettext not being able to work with ES2015
# template strings [1] by transpiling the JavaScript files in a special way to
# replace all template strings.
#
# [1] https://savannah.gnu.org/bugs/?50920


class Command(makemessages.Command):

    def handle(self, *args, **options):
        call_command("transpile")
        options['ignore_patterns'] += [
            'venv',
            'node_modules',
            'static-transpile'
        ]
        options['domain'] = 'django'
        super().handle(*args, **options)
        options['domain'] = 'djangojs'
        self.temp_dir_out = tempfile.mkdtemp()
        self.temp_dir_in = tempfile.mkdtemp()
        super().handle(*args, **options)
        shutil.rmtree(self.temp_dir_in)
        shutil.rmtree(self.temp_dir_out)

    def process_locale_dir(self, locale_dir, files):
        if self.domain == 'djangojs':
            for file in files:
                # We need to copy the JS files first, as otherwise babel will
                # attempt to read package.json files in subdirs, such as
                # base/package.json
                in_path = urllib.parse.urljoin(
                    self.temp_dir_in + '/',
                    file.dirpath
                )
                os.makedirs(in_path, exist_ok=True)
                in_file = urllib.parse.urljoin(
                    in_path + '/',
                    file.file
                )
                shutil.copy2(file.path, in_file)
                out_path = urllib.parse.urljoin(
                    self.temp_dir_out + '/',
                    file.dirpath
                )
                file.dirpath = out_path
            os.chdir('.transpile/')
            out, err, status = popen_wrapper([
                'npm',
                'run',
                'babel-transform-template-literals',
                '--',
                '--out-dir',
                self.temp_dir_out,
                self.temp_dir_in
            ])
            os.chdir('../')

        super().process_locale_dir(locale_dir, files)

    def write_po_file(self, potfile, locale):
        if self.domain == 'djangojs':
            with open(potfile, encoding='utf-8') as fp:
                msgs = fp.read()
            # Remove temp dir path info
            msgs = msgs.replace(self.temp_dir_out+"/", "")
            with open(potfile, 'w', encoding='utf-8') as fp:
                fp.write(msgs)
        super().write_po_file(potfile, locale)



# Generated by Django 2.2.2 on 2019-06-23 14:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0007_auto_20190227_2105'),
    ]

    operations = [
        migrations.AddField(
            model_name='documenttemplate',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.0, max_digits=3),
        ),
    ]



import os
import distutils
import setuptools
from setuptools.command.sdist import sdist as _sdist


def read(name):
    with open(
        os.path.join(os.path.dirname(os.path.realpath(__file__)), name)
    ) as f:
        return f.read()


class compilemessages(distutils.cmd.Command):
    """A custom command to create *.mo files for each language."""
    description = "Create *.mo files to be included in the final package."
    user_options = []

    def initialize_options(self):
        """Set default values for options."""
        pass

    def finalize_options(self):
        """Post-process options."""
        pass

    def run(self):
        import subprocess
        command = []
        command.append(os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'fiduswriter/manage.py'
        ))
        command.append('compilemessages')
        self.announce(
            'Running command: %s' % str(' '.join(command)),
            level=distutils.log.INFO
        )
        subprocess.check_call(command)


class sdist(_sdist):
    """Custom build command."""

    def run(self):
        self.run_command('compilemessages')
        _sdist.run(self)

cmdclass = {
    'compilemessages': compilemessages,
    'sdist': sdist
}

try:
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
    class bdist_wheel(_bdist_wheel):
        """Custom build command."""

        def run(self):
            self.run_command('compilemessages')
            _bdist_wheel.run(self)
    cmdclass['bdist_wheel'] = bdist_wheel
except ImportError:
    pass


setuptools.setup(
    cmdclass=cmdclass,
    name="fiduswriter",
    version=read('version.txt').splitlines()[0],
    description="A semantic wordprocessor for academic purposes",
    license="AGPL",
    author="Lund Info AB",
    author_email="mail@lundinfo.com",
    url="https://www.fiduswriter.org",
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "Framework :: Django :: 2.2",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering",
        "Topic :: Text Editors :: Word Processors",
        "Topic :: Text Processing",
        "Topic :: Text Processing :: Markup :: HTML",
        "Topic :: Text Processing :: Markup :: LaTeX",
        "Topic :: Text Processing :: Markup :: XML",
        "Topic :: Utilities",
    ],
    packages=setuptools.find_packages(),
    include_package_data=True,
    python_requires='>=3',
    install_requires=read('fiduswriter/requirements.txt').splitlines(),
    extras_require={
        "books": "fiduswriter-books ~= 3.7.1",
        "citation-api-import": "fiduswriter-citation-api-import ~= 3.7.0",
        "languagetool": "fiduswriter-languagetool ~= 3.7.0",
        "ojs": "fiduswriter-ojs ~= 3.7.0",
        "phplist": "fiduswriter-phplist ~= 3.7.0"
    },
    entry_points={
        "console_scripts": [
            "fiduswriter=fiduswriter.manage:entry"
        ]
    }
)



# Generated by Django 2.2.4 on 2019-08-31 14:09
from django.db import migrations, models
from django.utils.text import slugify


def set_import_id(apps, schema_editor):
    DocumentTemplate = apps.get_model('document', 'DocumentTemplate')
    for dt in DocumentTemplate.objects.all():
        title = dt.title
        if len(title) == 0:
            title = 'default'
        dt.import_id = slugify(title, allow_unicode=False)
        dt.save()


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0017_remove_documenttemplate_citation_styles'),
    ]

    operations = [
        migrations.AddField(
            model_name='documenttemplate',
            name='import_id',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.RunPython(set_import_id),
    ]



import os
from django.core.management.commands.compilemessages import (
    Command as CompilemessagesCommand
)


class Command(CompilemessagesCommand):
    def handle(self, *args, **options):
        if os.environ.get('NO_COMPILEMESSAGES'):
            self.stdout.write(
                'Using packaged version. Skipping compile messages.'
            )
        else:
            return super(Command, self).handle(*args, **options)
